@@ -12,6 +12,7 @@
 
 from unittest import mock
 from unittest.mock import call
+import uuid
 
 from osc_lib.cli import format_columns
 from osc_lib import exceptions
@@ -23,8 +24,13 @@ from openstackclient.tests.unit.network.v2 import fakes as network_fakes
 from openstackclient.tests.unit import utils as test_utils
 
 
-LIST_FIELDS_TO_RETRIEVE = ('id', 'name', 'mac_address', 'fixed_ips', 'status')
-LIST_FIELDS_TO_RETRIEVE_LONG = ('security_group_ids', 'device_owner', 'tags')
+LIST_FIELDS_TO_RETRIEVE = ['id', 'name', 'mac_address', 'fixed_ips', 'status']
+LIST_FIELDS_TO_RETRIEVE_LONG = [
+    'security_groups',
+    'device_owner',
+    'tags',
+    'trunk_details',
+]
 
 
 class TestPort(network_fakes.TestNetworkV2):
@@ -73,6 +79,7 @@ class TestPort(network_fakes.TestNetworkV2):
             'security_group_ids',
             'status',
             'tags',
+            'trusted',
             'trunk_details',
             'updated_at',
         )
@@ -114,6 +121,7 @@ class TestPort(network_fakes.TestNetworkV2):
             format_columns.ListColumn(fake_port.security_group_ids),
             fake_port.status,
             format_columns.ListColumn(fake_port.tags),
+            fake_port.trusted,
             fake_port.trunk_details,
             fake_port.updated_at,
         )
@@ -330,7 +338,7 @@ class TestCreatePort(TestPort):
                 self._port.network_id,
             ),
             ('enable', True),
-            ('security_group', [secgroup.id]),
+            ('security_groups', [secgroup.id]),
             ('name', 'test-port'),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
@@ -403,7 +411,7 @@ class TestCreatePort(TestPort):
                 self._port.network_id,
             ),
             ('enable', True),
-            ('security_group', [sg_1.id, sg_2.id]),
+            ('security_groups', [sg_1.id, sg_2.id]),
             ('name', 'test-port'),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
@@ -432,7 +440,7 @@ class TestCreatePort(TestPort):
         verifylist = [
             ('network', self._port.network_id),
             ('enable', True),
-            ('no_security_group', True),
+            ('security_groups', []),
             ('name', 'test-port'),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
@@ -1111,6 +1119,50 @@ class TestCreatePort(TestPort):
     def test_create_with_hardware_offload_type_null(self):
         self._test_create_with_hardware_offload_type()
 
+    def _test_create_with_trusted_field(self, trusted):
+        arglist = [
+            '--network',
+            self._port.network_id,
+            'test-port',
+        ]
+        if trusted:
+            arglist += ['--trusted']
+        else:
+            arglist += ['--not-trusted']
+
+        verifylist = [
+            (
+                'network',
+                self._port.network_id,
+            ),
+            ('name', 'test-port'),
+        ]
+        if trusted:
+            verifylist.append(('trusted', True))
+        else:
+            verifylist.append(('trusted', False))
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        columns, data = self.cmd.take_action(parsed_args)
+
+        create_args = {
+            'admin_state_up': True,
+            'network_id': self._port.network_id,
+            'name': 'test-port',
+        }
+        create_args['trusted'] = trusted
+        self.network_client.create_port.assert_called_once_with(**create_args)
+
+        self.assertEqual(set(self.columns), set(columns))
+        self.assertCountEqual(self.data, data)
+
+    def test_create_with_trusted_true(self):
+        self._test_create_with_trusted_field(True)
+
+    def test_create_with_trusted_false(self):
+        self._test_create_with_trusted_field(False)
+
 
 class TestDeletePort(TestPort):
     # Ports to delete.
@@ -1190,17 +1242,47 @@ class TestDeletePort(TestPort):
 
 
 class TestListPort(compute_fakes.FakeClientMixin, TestPort):
-    _ports = network_fakes.create_ports(count=3)
+    _project = identity_fakes.FakeProject.create_one_project()
+    _networks = network_fakes.create_networks(count=3)
+    _sport1 = network_fakes.create_one_port(
+        attrs={'project_id': _project.id, 'network_id': _networks[1]['id']}
+    )
+    _sport2 = network_fakes.create_one_port(
+        attrs={'project_id': _project.id, 'network_id': _networks[2]['id']}
+    )
+    _trunk_details = {
+        'trunk_id': str(uuid.uuid4()),
+        'sub_ports': [
+            {
+                'segmentation_id': 100,
+                'segmentation_type': 'vlan',
+                'port_id': _sport1.id,
+            },
+            {
+                'segmentation_id': 102,
+                'segmentation_type': 'vlan',
+                'port_id': _sport2.id,
+            },
+        ],
+    }
+    _pport = network_fakes.create_one_port(
+        attrs={
+            'project_id': _project.id,
+            'network_id': _networks[0]['id'],
+            'trunk_details': _trunk_details,
+        }
+    )
+    _ports = (_pport, _sport1, _sport2)
 
-    columns = (
+    columns = [
         'ID',
         'Name',
         'MAC Address',
         'Fixed IP Addresses',
         'Status',
-    )
+    ]
 
-    columns_long = (
+    columns_long = [
         'ID',
         'Name',
         'MAC Address',
@@ -1209,7 +1291,8 @@ class TestListPort(compute_fakes.FakeClientMixin, TestPort):
         'Security Groups',
         'Device Owner',
         'Tags',
-    )
+        'Trunk subports',
+    ]
 
     data = []
     for prt in _ports:
@@ -1235,6 +1318,7 @@ class TestListPort(compute_fakes.FakeClientMixin, TestPort):
                 format_columns.ListColumn(prt.security_group_ids),
                 prt.device_owner,
                 format_columns.ListColumn(prt.tags),
+                port.SubPortColumn(prt.trunk_details),
             )
         )
 
@@ -2021,7 +2105,7 @@ class TestSetPort(TestPort):
             self._port.name,
         ]
         verifylist = [
-            ('security_group', [sg.id]),
+            ('security_groups', [sg.id]),
             ('port', self._port.name),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
@@ -2054,7 +2138,7 @@ class TestSetPort(TestPort):
             _testport.name,
         ]
         verifylist = [
-            ('security_group', [sg_2.id, sg_3.id]),
+            ('security_groups', [sg_2.id, sg_3.id]),
             ('port', _testport.name),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
@@ -2103,7 +2187,7 @@ class TestSetPort(TestPort):
             _testport.name,
         ]
         verifylist = [
-            ('security_group', [sg2.id]),
+            ('security_groups', [sg2.id]),
             ('no_security_group', True),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
@@ -2520,6 +2604,35 @@ class TestSetPort(TestPort):
         )
         self.assertIsNone(result)
 
+    def _test_set_trusted_field(self, trusted):
+        arglist = [self._port.id]
+        if trusted:
+            arglist += ['--trusted']
+        else:
+            arglist += ['--not-trusted']
+
+        verifylist = [
+            ('port', self._port.id),
+        ]
+        if trusted:
+            verifylist.append(('trusted', True))
+        else:
+            verifylist.append(('trusted', False))
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        result = self.cmd.take_action(parsed_args)
+        self.network_client.update_port.assert_called_once_with(
+            self._port, **{'trusted': trusted}
+        )
+        self.assertIsNone(result)
+
+    def test_set_trusted_true(self):
+        self._test_set_trusted_field(True)
+
+    def test_set_trusted_false(self):
+        self._test_set_trusted_field(False)
+
 
 class TestShowPort(TestPort):
     # The port to show.
@@ -2695,7 +2808,7 @@ class TestUnsetPort(TestPort):
             _fake_port.name,
         ]
         verifylist = [
-            ('security_group_ids', [_fake_sg2.id]),
+            ('security_groups', [_fake_sg2.id]),
         ]
 
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
@@ -2722,7 +2835,7 @@ class TestUnsetPort(TestPort):
             _fake_port.name,
         ]
         verifylist = [
-            ('security_group_ids', [_fake_sg2.id]),
+            ('security_groups', [_fake_sg2.id]),
         ]
 
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
